using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

using wclCommon;
using wclCommunication;
using wclBluetooth;

namespace ClientTest
{
    class Program
    {
        private class Thread : wclThread
        {
            private wclBluetoothManager Manager;
            private wclRfCommClient Client;
            private List<Int64> Devices;
            private ManualResetEvent CompleteEvent;

            protected override bool OnInitialize()
            {
                Manager = new wclBluetoothManager();
                Manager.OnAuthenticationCompleted += new wclBluetoothDeviceResultEvent(Manager_OnAuthenticationCompleted);
                Manager.OnNumericComparison += new wclBluetoothNumericComparisonEvent(Manager_OnNumericComparison);
                Manager.OnPasskeyNotification += new wclBluetoothPasskeyNotificationEvent(Manager_OnPasskeyNotification);
                Manager.OnPasskeyRequest += new wclBluetoothPasskeyRequestEvent(Manager_OnPasskeyRequest);
                Manager.OnPinRequest += new wclBluetoothPinRequestEvent(Manager_OnPinRequest);
                Manager.OnDiscoveringStarted += new wclBluetoothEvent(Manager_OnDiscoveringStarted);
                Manager.OnDeviceFound += new wclBluetoothDeviceEvent(Manager_OnDeviceFound);
                Manager.OnDiscoveringCompleted += new wclBluetoothResultEvent(Manager_OnDiscoveringCompleted);

                Int32 Res = Manager.Open();
                if (Res != wclErrors.WCL_E_SUCCESS)
                {
                    Console.WriteLine("Failed to open Bluetooth Manager: 0x" + Res.ToString("X8"));
                    goto ExitError;
                }

                if (Manager.Count == 0)
                {
                    Console.WriteLine("No one Bluetooth Radio was found");
                    goto ExitCloseManager;
                }

                wclBluetoothRadio Radio = null;
                for (int i = 0; i < Manager.Count; i++)
                {
                    if (Manager[i].Available)
                    {
                        Radio = Manager[i];
                        break;
                    }
                }

                if (Radio == null)
                {
                    Console.WriteLine("No available Bluetooth Radio was found");
                    goto ExitCloseManager;
                }

                Res = Radio.Discover(10, wclBluetoothDiscoverKind.dkClassic);
                Radio = null;
                if (Res != wclErrors.WCL_E_SUCCESS)
                {
                    Console.WriteLine("Failed to start discovering: 0x" + Res.ToString("X8"));
                    goto ExitCloseManager;
                }

                return true;

            ExitCloseManager:
                Manager.Close();

            ExitError:
                Manager = null;
                return false;
            }

            void Manager_OnDiscoveringCompleted(object Sender, wclBluetoothRadio Radio, int Error)
            {
                Console.WriteLine("Discovering completed wirh result: 0x" + Error.ToString("X8"));

                if (Error != wclErrors.WCL_E_SUCCESS)
                    CompleteEvent.Set();
                else
                {
                    if (Devices.Count == 0)
                    {
                        Console.WriteLine("No Bluetooth devices were found");
                        CompleteEvent.Set();
                    }
                    else
                    {
                        Boolean Connected = false;
                        foreach (Int64 Address in Devices)
                        {
                            wclBluetoothService[] Services = null;
                            Int32 Res = Radio.EnumRemoteServices(Address, wclUUIDs.SerialPortServiceClass_UUID, out Services);
                            if (Res == wclErrors.WCL_E_SUCCESS)
                            {
                                if (Services != null && Services.Length > 0)
                                {
                                    Client = new wclRfCommClient();
                                    Client.OnConnect += new wclClientConnectionConnectEvent(Client_OnConnect);
                                    Client.OnDisconnect += new wclClientConnectionDisconnectEvent(Client_OnDisconnect);
                                    Client.OnData += new wclConnectionDataEvent(Client_OnData);

                                    Client.Address = Address;
                                    Res = Client.Connect(Radio);
                                    if (Res == wclErrors.WCL_E_SUCCESS)
                                    {
                                        Connected = true;
                                        break;
                                    }
                                    else
                                        Client = null;
                                }
                            }

                            if (Connected)
                                break;
                        }

                        if (!Connected)
                        {
                            Console.WriteLine("No devices with SPP were found");
                            CompleteEvent.Set();
                        }
                    }
                }
            }

            void Client_OnData(object Sender, byte[] Data)
            {
                if (Data != null)
                {
                    String s = Encoding.ASCII.GetString(Data);
                    Console.WriteLine("Received: " + s);
                }
            }

            void Client_OnDisconnect(object Sender, int Reason)
            {
                Console.WriteLine("Disconnected");
            }

            void Client_OnConnect(object Sender, int Error)
            {
                Console.WriteLine("Connected with result: 0x" + Error.ToString("X8"));
                CompleteEvent.Set();
            }

            void Manager_OnDeviceFound(object Sender, wclBluetoothRadio Radio, long Address)
            {
                Console.WriteLine("Device " + Address.ToString("X12") + " found");
                Devices.Add(Address);
            }

            void Manager_OnDiscoveringStarted(object Sender, wclBluetoothRadio Radio)
            {
                Console.WriteLine("Discovering started");
                Devices = new List<Int64>();
            }

            void Manager_OnPinRequest(object Sender, wclBluetoothRadio Radio, long Address, out string Pin)
            {
                Console.WriteLine("PIN request. Use PIN: 0000");
                Pin = "0000";
            }

            void Manager_OnPasskeyRequest(object Sender, wclBluetoothRadio Radio, long Address, out uint Passkey)
            {
                Console.WriteLine("Passkey request. Type the passkey");
                Passkey = Convert.ToUInt32(Console.ReadLine());
            }

            void Manager_OnPasskeyNotification(object Sender, wclBluetoothRadio Radio, long Address, uint Passkey)
            {
                Console.WriteLine("Passkey notification: " + Passkey.ToString());
            }

            void Manager_OnNumericComparison(object Sender, wclBluetoothRadio Radio, long Address, uint Number, out bool Confirm)
            {
                Console.WriteLine("Numeri comparison: " + Number.ToString());
                Confirm = true;
            }

            void Manager_OnAuthenticationCompleted(object Sender, wclBluetoothRadio Radio, long Address, int Error)
            {
                Console.WriteLine("Authentication completed with result: 0x" + Error.ToString("X8"));
            }

            protected override void OnTerminate()
            {
                if (Client != null)
                {
                    Client.Disconnect();
                    Client = null;
                }

                if (Manager != null)
                {
                    Manager.Close();
                    Manager = null;
                }

                if (Devices != null)
                {
                    Devices.Clear();
                    Devices = null;
                }

                base.OnTerminate();
            }

            public Thread(ManualResetEvent Event) : base()
            {
                Manager = null;
                Client = null;
                Devices = null;
                CompleteEvent = Event;
            }
        };

        static void Main(string[] args)
        {
            ManualResetEvent Event = new ManualResetEvent(false);
            Thread Thread = new Thread(Event);
            if (Thread.Run() == wclErrors.WCL_E_SUCCESS)
            {
                Event.WaitOne();
                Console.WriteLine("Press any key to disconnect");
                Console.ReadKey();
                Thread.Terminate();
            }
            Thread = null;
            Event.Close();
            Event = null;
        }
    }
}
